/////////////////////////////////////////////////////////////////////////////
// StandardShader_D3D11.fx
//
// Copyright (c) 2006, Electronic Arts Inc. All rights reserved.
// Written by Alex Liberman (based on Avery Lee's StandardShader code)
//
// StandardShader is a D3D11/Capilano implementation of the UTFWin 
// StandardShader2D.
//
// vertex shader: Transforms vertices according to UTFWin's transformation
// pipeline (bypassing the D3D pipeline). Also modulates the vertex color 
// by the global shade (which is set on a per-part basis).
//
// pixel shader: Modulates the incoming window color by an associated
// texture (if present), otherwise uses the window color
//
/////////////////////////////////////////////////////////////////////////////


//--------------------------------------------------------------------------------------
// Constant Buffer Variables
//--------------------------------------------------------------------------------------
Texture2D tex2D;

SamplerState samLinear;

cbuffer cSettingsBuffer : register( b0 )
{
    matrix MVP; 
    float4 GlobalShade; 
    float TextureEnable; 
    float3 pad;              
};

cbuffer cDataBuffer : register( b1 )
{
    float4 StandardShaderData[80];           
};

//--------------------------------------------------------------------------------------
struct VS_INPUT
{
    float4 Pos : POSITION;
    float4 Color0 : COLOR0;
    float2 Tex : TEXCOORD0;
	float4 Color1 : COLOR1;
};

struct PS_INPUT
{
    float4 Pos : SV_POSITION;
    float4 Color : COLOR0;
    float3 Tex : TEXCOORD0;
};


//--------------------------------------------------------------------------------------
// Vertex Shader
//--------------------------------------------------------------------------------------
PS_INPUT VSBasic( VS_INPUT input )
{
    PS_INPUT output = (PS_INPUT)0;
    output.Pos = mul( input.Pos, MVP );
    output.Color = input.Color0;
    output.Tex = float3(input.Tex, 0.0f);     // Z-component ignored in non-batched pixel shader
    return output;
}

PS_INPUT VSBatched( VS_INPUT input )
{
    PS_INPUT output = (PS_INPUT)0;
    float4x4 mvpIndex;

    output.Tex	= float3(input.Tex, input.Color1.b);    // TextureEnable flag is in the BLUE channel of the selector
    float idx   = input.Color1.g * 255.1;

    // StandardShaderData holds data for 16 windows. Each datum consists of 5 floats: 1 float for the 
    // global shade color, remaining 4 floats comprise the UTFWin object-to-screen transform
    output.Color = input.Color0 * StandardShaderData[idx];   // Modulate window color by the global shade
    mvpIndex     = float4x4(StandardShaderData[idx+1], StandardShaderData[idx+2], StandardShaderData[idx+3], StandardShaderData[idx+4]);    // Extract UTFWin transform at given index
    output.Pos   = mul(input.Pos, mvpIndex);

	return output;
}

//--------------------------------------------------------------------------------------
// Pixel Shader
//--------------------------------------------------------------------------------------
float4 PSBasic( PS_INPUT input) : SV_Target
{
    float4 ct = input.Color * tex2D.Sample( samLinear, input.Tex.xy);
	float4 c = lerp(input.Color, ct, TextureEnable);
	return c;    
}


float4 PSBatched( PS_INPUT input) : SV_Target
{
	float4 ct = input.Color * tex2D.Sample( samLinear, input.Tex.xy);
	float4 c = lerp(input.Color, ct, input.Tex.z);
	return c;
}